from common import *
from scitools.numpyutils import ravel, zeros, array, allclose, rank, \
     meshgrid, newaxis
from scitools.globaldata import DEBUG, VERBOSE
from scitools.numpytools import NumPy_dtype
from scitools.misc import test_if_module_exists 

test_if_module_exists('vtk', msg='You need to install the VTK package.', abort=False)
import vtk
#import vtk.util.colors
import os

import Tkinter
# use old behavior in Tkinter module to get around issue with Tcl
# (more info: http://www.python.org/doc/2.3/whatsnew/node18.html)
Tkinter.wantobjects = 0

try: import vtkTkRenderWidget
except:
    from vtk.tk import vtkTkRenderWidget

class VtkBackend(BaseClass):
    """Backend using VTK."""

    def __init__(self):
        BaseClass.__init__(self)
        self.init()

    def init(self):
        self._master = None
        self.figure(self._attrs['curfig'])

        # conversion tables for format strings
        self._colors = {
            '':  (0,0,1), # No color-->Blue
            'k': (0,0,0), # Black
            'r': (1,0,0), # Red 
            'g': (0,1,0), # Green
            'b': (0,0,1), # Blue
            'm': (1,0,1), # Magenta
            'c': (0,1,1), # Cyan
            'w': (1,1,1), # White
            'y': (1,1,0), # Yellow
            }

        self._arrow_types = { # tuple: (type,rotation)
            '':  (9,0),  # arrow
            '.': (0,0),  # no marker
            'o': (7,0),  # circle
            '+': (3,0),  # plus
            'x': (3,45), # x-mark
            '*': (3,0),  # star --> plus
            's': (6,0),  # square
            'd': (8,0),  # diamond
            'v': (5,180),# triangle (down) 
            '^': (5,0),  # triangle (up)
            '<': (5,90), # triangle (left) 
            '>': (5,270),# triangle (right)
            'p': (6,0),  # pentagram --> square
            'h': (6,0),  # hexagram --> square
            }

        self._colorbar_locations = {
            'North': ((.2, .75), (.6,.09)),
            'South': ((.2, .2), (.6, .09)),
            'East': ((.75, .09), (.1, .9)),
            'West': ((.2, .09), (.1, .9)),
            'NorthOutside': ((.2, .86), (.6,.09)),
            'SouthOutside': ((.2, .06), (.6, .09)),
            'EastOutside': ((.86, .09), (.1, .9)),
            'WestOutside': ((.01, .09), (.1, .9))
            }

        try:
            v = vtk.vtkMesaRenderer()
            _graphics_fact = vtk.vtkGraphicsFactory()
            _graphics_fact.SetUseMesaClasses(1)
            _image_fact = vtk.vtkImagingFactory()
            _image_fact.SetUseMesaClasses(1)
            del _graphics_fact
            del _image_fact
            del v
        except Exception, msg:
            if DEBUG:
                print "No mesa", msg

    def _create_Tk_gui(self):
        fig = self.gcf()
        if self._master is None:
            self._master = Tkinter.Tk()
            self._master.withdraw()
        fig._root = Tkinter.Toplevel(self._master)
        fig._root.title("Easyviz VTK Data Visualizer - Figure %d" % \
                        self._attrs['curfig'])
        # if the window is closed, we should delete the current figure and
        # create a new one.
        def _close_fig(event=None):
            self.clf()
            fig._root.withdraw()
        fig._root.protocol("WM_DELETE_WINDOW", _close_fig)
        fig._root.minsize(200, 200)
        fig._root.bind("<KeyPress-q>", _close_fig)
        fig._root.withdraw()
        master_f = Tkinter.Frame(fig._root, relief='sunken', bd=2)
        master_f.pack(side='top', fill='both', expand=1)
        renwin_frame = Tkinter.Frame(master_f)
        renwin_frame.pack(side='left', fill='both', expand=1)
        frame = Tkinter.Frame(renwin_frame)
        frame.pack(side='top', fill='both', expand=1)
        width, height = fig.getp('size')
        if width is None or height is None:
            width = 640;  height = 480
        tkw = vtkTkRenderWidget.vtkTkRenderWidget(frame,
                                                  width=width,
                                                  height=height)
        tkw.pack(expand='true', fill='both')
        renwin = tkw.GetRenderWindow()
        renwin.SetSize(width, height)
        #renwin.SetSize(width+1, height+1)
        #renwin.SetSize(width-1, height-1)
        #renwin.LineSmoothingOn()
        #tkw.UpdateRenderer(0.0, 0.0)
        #renwin.Render()
        return renwin

    def _set_view_old(self):
        axis_cam = self._axis.getp('camera')
        if not hasattr(self._axis, '_vtk_camera'):
            # initialize camera:
            pass
        else:
            # alter camera:
            pass
        camera = vtk.vtkCamera()
        self._axis._renderer.SetActiveCamera(camera)
        if axis_cam.getp('camproj') == 'perspective':
            camera.ParallelProjectionOff()
        else:
            camera.ParallelProjectionOn()
        fp = axis_cam.getp('camtarget')
        camera.SetFocalPoint(fp)
        camera.SetViewUp(axis_cam.getp('camup'))
        if axis_cam.getp('cammode') == 'auto':
            if axis_cam.getp('view') == 3:
                camera.SetPosition(fp[0],fp[1]-1,fp[2])
                camera.Azimuth(-37.5)
                camera.Elevation(30)
            else:
                camera.SetPosition(fp[0], fp[1], 1)
        else:
            camera.SetPosition(axis_cam.getp('campos'))
        #camera.ComputeViewPlaneNormal()
        #camera.OrthogonalizeViewUp()
        if axis_cam.getp('camva') is not None:
            camera.SetViewAngle(axis_cam.getp('camva'))
        azimuth = axis_cam.getp('azimuth')
        if azimuth is not None:
            if axis_cam.getp('view') == 3:
                azimuth += 37.5 # compensate for above
            camera.Azimuth(azimuth)
        elevation = axis_cam.getp('elevation')
        if elevation is not None:
            if axis_cam.getp('view') == 3:
                elevation -= 30 # compensate for above
            camera.Elevation(elevation)
        # make all actors fit inside the current scene:
        self._axis._renderer.ResetCamera()
        camera.Zoom(axis_cam.getp('camzoom'))
        self._axis._vtk_camera = camera

    def _initialize_camera(self):
        ax_cam = self._axis.getp('camera')
        camera = vtk.vtkCamera()
        fp = ax_cam.getp('camtarget')
        camera.SetFocalPoint(fp)
        camera.SetViewUp(ax_cam.getp('camup'))
        if ax_cam.getp('cammode') == 'auto':
            if ax_cam.getp('view') == 3:
                camera.SetPosition(fp[0],fp[1]-1,fp[2])
                camera.Azimuth(-37.5)
                camera.Elevation(30)
            else:
                camera.SetPosition(fp[0], fp[1], 1)
        else:
            camera.SetPosition(ax_cam.getp('campos'))
        azimuth = ax_cam.getp('azimuth')
        if azimuth is not None:
            if ax_cam.getp('view') == 3:
                azimuth += 37.5 # compensate for above
            camera.Azimuth(azimuth)
        elevation = ax_cam.getp('elevation')
        if elevation is not None:
            if ax_cam.getp('view') == 3:
                elevation -= 30 # compensate for above
            camera.Elevation(elevation)

        return camera

    def _update_camera(self):
        ax_cam = self._axis.getp('camera')
        camera = self._axis._vtk_camera
        return camera

    def _set_view(self):
        ax_cam = self._axis.getp('camera')
        if not hasattr(self._axis, '_vtk_camera'):
            camera = self._initialize_camera()
        else:
            #camera = self._update_camera()
            camera = self._initialize_camera()

        if ax_cam.getp('camroll') is not None:
            camera.Roll(ax_cam.getp('camroll'))

        if ax_cam.getp('camva') is not None:
            camera.SetViewAngle(ax_cam.getp('camva'))
 
        if ax_cam.getp('camproj') == 'perspective':
            camera.ParallelProjectionOff()
        else:
            camera.ParallelProjectionOn()
            
        self._axis._renderer.SetActiveCamera(camera)
        self._axis._vtk_camera = camera

        # make sure all actors are inside the current view:
        ren = self._axis._renderer
        ren.ResetCamera()
        #if self._axis.getp('camera').getp('view') == 2:
        #    ren.GetActiveCamera().Zoom(1.5)
        camera.Zoom(ax_cam.getp('camzoom'))
       
        # set the camera in the vtkCubeAxesActor2D object:
        self._axis._vtk_axes.SetCamera(camera)

    def _create_labeled_axes(self):
        ax = self._axis
        if not hasattr(ax, '_vtk_axes'):
            ax._vtk_axes = vtk.vtkCubeAxesActor2D()
        if ax.getp('visible'):
            tprop = vtk.vtkTextProperty()
            tprop.SetColor(ax.getp('fgcolor'))
            tprop.SetFontSize(ax.getp('fontsize'))
            tprop.SetShadow(0)
            tprop.SetBold(0)
            mode = ax.getp('mode')
            if mode in ['auto', 'tight']:
                dar = ax.getp('daspect')
                bounds = b = list(ax._vtk_apd.GetOutput().GetBounds())
                if mode == 'auto':
                    incr = 0.1
                    dx = float(b[1] - b[0])
                    dy = float(b[3] - b[2])
                    dz = float(b[5] - b[4])
                    #b[0] -= dx*incr;  b[1] += dx*incr
                    #b[2] -= dy*incr;  b[3] += dy*incr
                    #b[4] -= dz*incr;  b[5] += dz*incr
                unscaled_bounds = ub = bounds[:]
                ub[0] *= dar[0];  ub[1] *= dar[0]
                ub[2] *= dar[1];  ub[3] *= dar[1]
                ub[4] *= dar[2];  ub[5] *= dar[2]
            elif mode == 'fill':
                print "axis mode 'fill' not implemented in VtkBackend"
            elif mode == 'manual':
                bounds = ax._vtk_scaled_bounds
                unscaled_bounds = ax._vtk_bounds
            #cube_axes = vtk.vtkCubeAxesActor2D()
            ax._vtk_axes.SetBounds(bounds)
            ax._vtk_axes.SetRanges(unscaled_bounds)
            ax._vtk_axes.UseRangesOn()
            #ax._vtk_axes.SetCamera(ax._vtk_camera)
            ax._vtk_axes.SetLabelFormat("%6.3g")
            ax._vtk_axes.SetFlyModeToOuterEdges()
            #ax._vtk_axes.SetFontFactor(ax.getp('fontsize')/10.)
            ax._vtk_axes.ScalingOff()
            ax._vtk_axes.SetAxisTitleTextProperty(tprop)
            ax._vtk_axes.SetAxisLabelTextProperty(tprop)
            ax._vtk_axes.GetProperty().SetColor(ax.getp('fgcolor'))
            ax._vtk_axes.SetCornerOffset(0)
            ax._vtk_axes.SetNumberOfLabels(5)
            ax._vtk_axes.SetXLabel(ax.getp('xlabel'))
            ax._vtk_axes.SetYLabel(ax.getp('ylabel'))
            ax._vtk_axes.SetZLabel(ax.getp('zlabel'))
            if ax.getp('camera').getp('view') == 2:
                ax._vtk_axes.YAxisVisibilityOff()
            else:
                ax._vtk_axes.YAxisVisibilityOn()
            ax._renderer.AddActor(ax._vtk_axes)
            ax._vtk_box_bounds = bounds

    def _set_box_state(self):
        ax = self._axis
        if not hasattr(ax, '_vtk_box'):
            ax._vtk_box = vtk.vtkActor()
        # remove old box (if present):
        if ax._renderer.GetActors().IsItemPresent(ax._vtk_box):
            ax._renderer.RemoveActor(ax._vtk_box)
        if ax.getp('box'):
            box = vtk.vtkCubeSource()
            #box.SetBounds(ax._vtk_scaled_bounds)
            box.SetBounds(ax._vtk_box_bounds)
            box.Update()
            outline = vtk.vtkOutlineFilter()
            outline.SetInput(box.GetOutput())
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInput(outline.GetOutput())
            ax._vtk_box.SetMapper(mapper)
            ax._vtk_box.GetProperty().SetColor(ax.getp('fgcolor'))
            ax._renderer.AddActor(ax._vtk_box)

    def _set_colormap(self):
        colormap = self._axis.getp('colormap')
        if not isinstance(colormap, vtk.vtkLookupTable):
            colormap = self.jet() # use default colormap
        self._axis._vtk_colormap = colormap

    def _set_colorbar(self):
        ax = self._axis
        cbar = ax.getp('colorbar')

        if not hasattr(ax, '_vtk_colorbar'):
            ax._vtk_colorbar = vtk.vtkScalarBarActor()
        if ax._renderer.GetActors().IsItemPresent(ax._vtk_colorbar):
            ax._renderer.RemoveActor2D(ax._vtk_colorbar)
        if cbar.getp('visible'):
            cblocation = cbar.getp('cblocation')
            cbloc = self._colorbar_locations[cblocation]
            ax._vtk_colorbar.SetLookupTable(ax._vtk_colormap)
            if 'North' in cblocation or 'South' in cblocation:
                ax._vtk_colorbar.SetOrientationToHorizontal()
            else:
                ax._vtk_colorbar.SetOrientationToVertical()
            ax._vtk_colorbar.SetTitle(cbar.getp('cbtitle'))
            ax._vtk_colorbar.SetPosition(*cbloc[0])
            ax._vtk_colorbar.SetPosition2(*cbloc[1])
            tprop = vtk.vtkTextProperty()
            tprop.SetColor(ax.getp('fgcolor'))
            tprop.SetFontSize(ax.getp('fontsize'))
            tprop.ShadowOff()
            ax._vtk_colorbar.SetTitleTextProperty(tprop)
            ax._vtk_colorbar.SetLabelTextProperty(tprop)
            ax._renderer.AddActor(ax._vtk_colorbar)

    def _set_shading(self, item, source, actor):
        shading = self._axis.getp('shading')
        if shading == 'interp':
            actor.GetProperty().SetInterpolationToGouraud()
        elif shading == 'flat':
            actor.GetProperty().SetInterpolationToFlat()
        else: # use default shading ('faceted')
            actor.GetProperty().SetInterpolationToFlat()
            edges = vtk.vtkExtractEdges()
            edges.SetInput(source.GetOutput())
            edges.Update()
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInput(edges.GetOutput())
            mapper.ScalarVisibilityOff()
            mapper.SetResolveCoincidentTopologyToPolygonOffset()
            mesh = vtk.vtkActor()
            mesh.SetMapper(mapper)
            color = item.getp('linecolor')
            if color == '' or color is None:
                color = (0,0,0) # use black as default
            elif not isinstance(color, (tuple,list)):
                try: color = self._colors[color]
                except: color = (0,0,0) # use black as default
            mesh.GetProperty().SetColor(color)
            self._axis._renderer.AddActor(mesh)

    def _set_title(self):
        tprop = vtk.vtkTextProperty()
        tprop.BoldOff()
        tprop.SetFontSize(self._axis.getp('fontsize'))
        tprop.SetColor(self._axis.getp('fgcolor'))
        tprop.SetFontFamilyToArial()
        tprop.SetVerticalJustificationToTop()
        tprop.SetJustificationToCentered()
	mapper = vtk.vtkTextMapper()
        mapper.SetInput(self._fix_latex(self._axis.getp('title')))
        mapper.SetTextProperty(tprop)
	actor = vtk.vtkActor2D()
        actor.SetMapper(mapper)
        actor.GetPositionCoordinate().SetCoordinateSystemToView()
        actor.GetPositionCoordinate().SetValue(0.0, 0.95)
        self._axis._renderer.AddActor(actor)

    def _set_caxis(self):
        if self._axis.getp('caxismode') == 'auto':
            caxis = None
        else:
            caxis = self._axis.getp('caxis')
        self._axis._vtk_caxis = caxis
        
    def _data_inside_bounds(self, data):
        fb = self._axis._vtk_scaled_bounds
        bounds = data.GetBounds()
        for i in range(0, len(fb), 2):
            if bounds[i] < fb[i] and not allclose(bounds[i],fb[i]):
                return False
        for i in range(1, len(fb), 2):
            if bounds[i] > fb[i] and not allclose(bounds[i],fb[i]):
                return False
        return True

    def _cut_data(self, indata):
        if self._data_inside_bounds(indata.GetOutput()):
            return indata
        box = vtk.vtkBox()
        box.SetBounds(self._axis._vtk_scaled_bounds)
        clipper = vtk.vtkClipPolyData()
        clipper.SetInput(indata.GetOutput())
        clipper.SetClipFunction(box)
        #clipper.GenerateClipScalarsOn()
        #clipper.GenerateClippedOutputOn()
        clipper.SetValue(0.0)
        clipper.InsideOutOn()
        clipper.Update()
        return clipper

    def _add_slices(self, item, sgrid, contours=False):
        cvector = item.getp('cvector')
        center = sgrid.GetCenter()
        dar = self._axis.getp('daspect')
        sx, sy, sz = item.getp('slices')
        if len(shape(sx)) == 2 and shape(sx) == shape(sy) == shape(sz):
            s = Surface(sx,sy,sz)
            sgrid2 = self._get_2d_structured_grid(s)
            plane = vtk.vtkStructuredGridGeometryFilter()
            plane.SetInput(sgrid2)
            plane.Update()
            data = self._cut_data(plane)
            implds = vtk.vtkImplicitDataSet()
            implds.SetDataSet(data.GetOutput())
            implds.Modified()
            cut = vtk.vtkCutter()
            cut.SetInput(sgrid)
            cut.SetCutFunction(implds)
            cut.GenerateValues(10, -2,2)
            cut.GenerateCutScalarsOn()
            cut.Update()
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInput(cut.GetOutput())
            mapper.SetLookupTable(self._axis._vtk_colormap)
            caxis = self._axis.getp('caxis')
            if None in caxis:
                caxis = data.GetOutput().GetScalarRange()
            mapper.SetScalarRange(caxis)
            mapper.Update()
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            self._set_shading(item, data, actor)
            self._set_actor_properties(item, actor)
            self._axis._renderer.AddActor(actor)
            self._axis._vtk_apd.AddInput(cut.GetOutput())
            self._axis._vtk_apd.AddInput(data.GetOutput())
        else:
            origins = []
            normals = []
            sx = ravel(sx)/dar[0]
            sy = ravel(sy)/dar[1]
            sz = ravel(sz)/dar[2]
            for i in range(len(sx)):
                normals.append([1,0,0])
                origins.append([sx[i], center[1], center[2]])
            for i in range(len(sy)):
                normals.append([0,1,0])
                origins.append([center[0], sy[i], center[2]])
            for i in range(len(sz)):
                normals.append([0,0,1])
                origins.append([center[0], center[1], sz[i]])
            for i in range(len(normals)):
                plane = vtk.vtkPlane()
                plane.SetOrigin(origins[i])
                plane.SetNormal(normals[i])
                cut = vtk.vtkCutter()
                cut.SetInput(sgrid)
                cut.SetCutFunction(plane)
                cut.Update()
                data = self._cut_data(cut)
                mapper = vtk.vtkPolyDataMapper()
                if contours:
                    iso = vtk.vtkContourFilter()
                    iso.SetInput(data.GetOutput())
                    if cvector is not None:
                        for i in range(len(cvector)):
                            iso.SetValue(i, cvector[i])
                    else:
                        zmin, zmax = data.GetOutput().GetScalarRange()
                        iso.GenerateValues(item.getp('clevels'), zmin, zmax)
                    iso.Update()
                    mapper.SetInput(iso.GetOutput())
                else:
                    mapper.SetInput(data.GetOutput())
                mapper.SetLookupTable(self._axis._vtk_colormap)
                caxis = self._axis.getp('caxis')
                if None in caxis:
                    caxis = sgrid.GetScalarRange()
                mapper.SetScalarRange(caxis)
                mapper.Update()
                actor = vtk.vtkActor()
                actor.SetMapper(mapper)
                if not contours:
                    self._set_shading(item, data, actor)
                self._set_actor_properties(item, actor)
                self._axis._renderer.AddActor(actor)
                self._axis._vtk_apd.AddInput(cut.GetOutput())

    def _add_isosurface(self, item, sgrid):
        iso = vtk.vtkContourFilter()
        iso.SetInput(sgrid)
        iso.SetValue(0, item.getp('isovalue'))
        iso.Update()
        data = self._cut_data(iso)
        normals = vtk.vtkPolyDataNormals()
        normals.SetInput(data.GetOutput())
        normals.SetFeatureAngle(45)
        normals.Update()
        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInput(normals.GetOutput())
        mapper.SetLookupTable(self._axis._vtk_colormap)
        caxis = self._axis.getp('caxis')
        if None in caxis:
            caxis = sgrid.GetScalarRange()
        mapper.SetScalarRange(caxis)
        mapper.Update()
        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        self._set_shading(item, data, actor)
        self._set_actor_properties(item, actor)
        self._axis._renderer.AddActor(actor)
        self._axis._vtk_apd.AddInput(normals.GetOutput())

    def _get_2d_structured_grid(self, item, vectors=False,
                                heights=True, bottom=False):
        indexing = item.getp('indexing')
        dar = self._axis.getp('daspect')
        x = asarray(item.getp('xdata'))/dar[0]
        y = asarray(item.getp('ydata'))/dar[1]
        sgrid = vtk.vtkStructuredGrid()
        sgrid.SetDimensions(item.getp('dims'))
        no_points = item.getp('numberofpoints')
        points = vtk.vtkPoints()
        points.SetNumberOfPoints(no_points)
        if vectors:
            if hasattr(item, 'scale_vectors'):
                item.scale_vectors()
            x = ravel(x)
            y = ravel(y)
            u = asarray(item.getp('udata'))
            v = ravel(item.getp('vdata'))
            w = item.getp('wdata')
            z = item.getp('zdata')
            if z is None and w is None:
                z = w = zeros(shape(u))
            z = ravel(z)
            w = ravel(w)
            if item.getp('function') == 'quiver3':
                z = z/dar[2]
            if rank(u) == 2:
                nx, ny = shape(u)
                if indexing == 'ij':
                    if len(x) == nx:
                        x = ravel(x[:,newaxis]*ones((nx,ny)))
                    if len(y) == ny:
                        y = ravel(y[newaxis,:]*ones((nx,ny)))
                else:
                    if len(x) == ny:
                        x = ravel(x[newaxis,:]*ones((nx,ny)))
                    if len(y) == nx:
                        y = ravel(y[:,newaxis]*ones((nx,ny)))
            u = ravel(u)
            vectors = vtk.vtkFloatArray()
            vectors.SetNumberOfTuples(no_points)
            vectors.SetNumberOfComponents(3)
            vectors.SetNumberOfValues(3*no_points)
            assert shape(x)==shape(y)==shape(z)==shape(u)==shape(v)==shape(w),\
                   "matrix dimensions must agree"
            for i in range(no_points):
                points.SetPoint(i, x[i], y[i], z[i])
                vectors.SetTuple3(i, u[i], v[i], w[i])
            points.Modified()
            sgrid.SetPoints(points)
            sgrid.GetPointData().SetVectors(vectors)
        else:
            values = asarray(item.getp('zdata'))
            if heights:
                z = values/dar[2]
            elif bottom:
                z = zeros(values.shape, NumPy_dtype(values)) + \
                    self._axis._vtk_scaled_bounds[4]
            else:
                z = zeros(values.shape, NumPy_dtype(values))
            try:
                cdata = asarray(item.getp('cdata'))
            except KeyError:
                pass
            else:
                if cdata is not None and cdata.shape == values.shape:
                    values = cdata
            scalars = vtk.vtkFloatArray()
            scalars.SetNumberOfTuples(no_points)
            scalars.SetNumberOfComponents(1)
            nx, ny = shape(values)
            if not (shape(x) == shape(y) == (nx,ny)):
                x, y = meshgrid(ravel(x), ravel(y),
                                sparse=False, indexing=indexing)
            assert shape(x) == shape(y) == shape(z), \
                   "array dimensions must agree"
            ind = 0
            for j in range(ny):
                for i in range(nx):
                    points.SetPoint(ind, x[i,j], y[i,j], z[i,j])
                    scalars.SetValue(ind, values[i,j])
                    ind += 1
            points.Modified()
            sgrid.SetPoints(points)
            sgrid.GetPointData().SetScalars(scalars)

        sgrid.Update()
        return sgrid

    def _get_3d_structured_grid(self, item, vectors=False):
        indexing = item.getp('indexing')
        dar = self._axis.getp('daspect')
        x = asarray(item.getp('xdata'))/dar[0]
        y = asarray(item.getp('ydata'))/dar[1]
        z = asarray(item.getp('zdata'))/dar[2]
        sgrid = vtk.vtkStructuredGrid()
        sgrid.SetDimensions(item.getp('dims'))
        no_points = item.getp('numberofpoints')
        points = vtk.vtkPoints()
        points.SetNumberOfPoints(no_points)
        if vectors:
            u = asarray(item.getp('udata'))
            v = asarray(item.getp('vdata'))
            w = asarray(item.getp('wdata'))
            nx, ny, nz = shape(u)
            if not (shape(x) == shape(y) == shape(z)):
                x, y, z = meshgrid(ravel(x), ravel(y), ravel(z),
                                   sparse=False, indexing=indexing)
            assert shape(x) == shape(y) == shape(z) == \
                   shape(u) == shape(v) == shape(w), \
                   "array dimensions must agree"
                
            vectors = vtk.vtkFloatArray()
            vectors.SetNumberOfTuples(no_points)
            vectors.SetNumberOfComponents(3)
            vectors.SetNumberOfValues(3*no_points)

            ind = 0
            for k in range(nz):
                for j in range(ny):
                    for i in range(nx):
                        points.SetPoint(ind, x[i,j,k], y[i,j,k], z[i,j,k])
                        vectors.SetTuple3(ind, u[i,j,k], v[i,j,k], w[i,j,k])
                        ind += 1
            points.Modified()
            sgrid.SetPoints(points)
            sgrid.GetPointData().SetVectors(vectors)
        else:
            scalars = vtk.vtkFloatArray()
            scalars.SetNumberOfTuples(no_points)
            scalars.SetNumberOfComponents(1)
            
            v = asarray(item.getp('vdata'))
            # TODO: what about pseudocolor data?
            #cdata = ravel(item.getp('cdata'))
            #if cdata is not None:
            #    v = cdata
            nx, ny, nz = shape(v)
            if not (shape(x) == shape(y) == shape(z) == (nx,ny,nz)):
                x, y, z = meshgrid(ravel(x), ravel(y), ravel(z),
                                   sparse=False, indexing=indexing)
            assert shape(x) == shape(y) == shape(z) == shape(v), \
                   "array dimensions must agree"
            ind = 0
            for k in range(nz):
                for j in range(ny):
                    for i in range(nx):
                        points.SetPoint(ind, x[i,j,k], y[i,j,k], z[i,j,k])
                        scalars.SetValue(ind, v[i,j,k])
                        ind += 1
            points.Modified()
            sgrid.SetPoints(points)
            sgrid.GetPointData().SetScalars(scalars)
            
        sgrid.Update()
        return sgrid

    def _add_surface(self, item, sgrid):
        plane = vtk.vtkStructuredGridGeometryFilter()
        plane.SetInput(sgrid)
        plane.Update()
        data = self._cut_data(plane)
        normals = vtk.vtkPolyDataNormals()
        normals.SetInput(data.GetOutput())
        normals.SetFeatureAngle(45)
        normals.Update()
        mapper = vtk.vtkDataSetMapper()
        mapper.SetInput(normals.GetOutput())
        mapper.SetLookupTable(self._axis._vtk_colormap)
        caxis = self._axis.getp('caxis')
        if None in caxis:
            caxis = data.GetOutput().GetScalarRange()
        mapper.SetScalarRange(caxis)
        mapper.Update()
        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        if item.getp('wireframe'):
            actor.GetProperty().SetRepresentationToWireframe()
        else:
            self._set_shading(item, data, actor)
        self._set_actor_properties(item, actor)
        self._add_legend(item, normals.GetOutput())
        self._axis._renderer.AddActor(actor)
        self._axis._vtk_apd.AddInput(normals.GetOutput())
        
    def _add_contours(self, item, sgrid):
        plane = vtk.vtkStructuredGridGeometryFilter()
        plane.SetInput(sgrid)
        plane.Update()
        data = self._cut_data(plane)
        filled = item.getp('filled')
        if filled:
            iso = vtk.vtkBandedPolyDataContourFilter()
            iso.SetScalarModeToValue()
            iso.GenerateContourEdgesOn()
        else:
            iso = vtk.vtkContourFilter()
        iso.SetInput(data.GetOutput())
        clevels = item.getp('clevels')
        cvector = item.getp('cvector')
        if cvector is not None:
            for i in range(clevels):
                iso.SetValue(i, cvector[i])
        else:
            zmin, zmax = data.GetOutput().GetScalarRange()
            iso.SetNumberOfContours(clevels)
            iso.GenerateValues(clevels, zmin, zmax)
        iso.Update()
        isoMapper = vtk.vtkPolyDataMapper()
        isoMapper.SetInput(iso.GetOutput())
        isoMapper.SetLookupTable(self._axis._vtk_colormap)
        caxis = self._axis.getp('caxis')
        if None in caxis:
            caxis = data.GetOutput().GetScalarRange()
        isoMapper.SetScalarRange(caxis)
        if item.getp('linecolor'): # linecolor is defined:
            isoMapper.ScalarVisibilityOff()
        isoMapper.Update()
        isoActor = vtk.vtkActor()
        isoActor.SetMapper(isoMapper)
        self._set_actor_properties(item, isoActor)
        self._add_legend(item, iso.GetOutput())
        self._axis._renderer.AddActor(isoActor)
        self._axis._vtk_apd.AddInput(data.GetOutput())

        if filled: # create contour edges:
            edgeMapper = vtk.vtkPolyDataMapper()
            edgeMapper.SetInput(iso.GetContourEdgesOutput())
            edgeMapper.SetResolveCoincidentTopologyToPolygonOffset()
            edgeActor = vtk.vtkActor()
            edgeActor.SetMapper(edgeMapper)
            edgeActor.GetProperty().SetColor(0, 0, 0)
            self._axis._renderer.AddActor(edgeActor)

        if item.getp('clabels'):
            # subsample the points and label them:
            mask = vtk.vtkMaskPoints()
            mask.SetInput(iso.GetOutput())
            mask.SetOnRatio(data.GetOutput().GetNumberOfPoints()/50)
            mask.SetMaximumNumberOfPoints(50)
            mask.RandomModeOn()

            # Create labels for points - only show visible points
            visPts = vtk.vtkSelectVisiblePoints()
            visPts.SetInput(mask.GetOutput())
            visPts.SetRenderer(self._axis._renderer)
            ldm = vtk.vtkLabeledDataMapper()
            ldm.SetInput(mask.GetOutput())
            ldm.SetLabelFormat("%.1g")
            ldm.SetLabelModeToLabelScalars()
            tprop = ldm.GetLabelTextProperty()
            tprop.SetFontFamilyToArial()
            tprop.SetFontSize(10)
            tprop.SetColor(0,0,0)
            tprop.ShadowOff()
            tprop.BoldOff()
            contourLabels = vtk.vtkActor2D()
            contourLabels.SetMapper(ldm)
            self._axis._renderer.AddActor(contourLabels)

    def _add_velocity_vectors(self, item, sgrid):
        marker, rotation = self._arrow_types[item.getp('linemarker')]
        arrow = vtk.vtkGlyphSource2D()
        arrow.SetGlyphType(marker)
        arrow.SetFilled(item.getp('filledarrows'))
        arrow.SetRotationAngle(rotation)
        if arrow.GetGlyphType() != 9: # not an arrow
            arrow.DashOn()
            arrow.SetCenter(.75,0,0)
        else:
            arrow.SetCenter(.5,0,0)        
        arrow.SetColor(self._colors[item.getp('linecolor')])

        plane = vtk.vtkStructuredGridGeometryFilter()
        plane.SetInput(sgrid)
        plane.Update()
        data = self._cut_data(plane)
        glyph = vtk.vtkGlyph3D()
        glyph.SetInput(data.GetOutput()) 
        glyph.SetSource(arrow.GetOutput())
        glyph.SetColorModeToColorByVector()
        glyph.SetRange(data.GetOutput().GetScalarRange())
        glyph.ScalingOn()
        glyph.SetScaleModeToScaleByVector()
        glyph.OrientOn()
        glyph.SetVectorModeToUseVector()
        glyph.Update()
        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInput(glyph.GetOutput())
        #vr = data.GetOutput().GetPointData().GetVectors().GetRange()
        #mapper.SetScalarRange(vr)
        mapper.ScalarVisibilityOff()
        mapper.Update()
        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        self._set_actor_properties(item, actor)
        self._add_legend(item, arrow.GetOutput())
        self._axis._renderer.AddActor(actor)
        self._axis._vtk_apd.AddInput(glyph.GetOutput())

    def _add_streams(self, item, sgrid):
        length = sgrid.GetLength()
        max_velocity = sgrid.GetPointData().GetVectors().GetMaxNorm()
        max_time = 35.0*length/max_velocity
        
        n = item.getp('numberofstreams')
        sx = ravel(item.getp('startx'))
        sy = ravel(item.getp('starty'))
        sz = None
        if item.getp('startz') is not None:
            sz = ravel(item.getp('startz'))
        dar = self._axis.getp('daspect')
        for i in range(n):
            integ = vtk.vtkRungeKutta2() # or 4?
            stream = vtk.vtkStreamLine()
            stream.SetInput(sgrid)
            stream.SetStepLength(item.getp('stepsize'))
            #stream.SetIntegrationStepLength(item.getp('stepsize'))
            #stream.SetIntegrationDirectionToIntegrateBothDirections()
            stream.SetIntegrationDirectionToForward()
            #stream.SetMaximumPropagationTime(max_time)
            #stream.SetMaximumPropagationTime(200)
            stream.SpeedScalarsOn()
            #stream.VorticityOn()
            if sz is None:
                stream.SetStartPosition(sx[i]/dar[0], sy[i]/dar[1], 0)
            else:
                stream.SetStartPosition(sx[i]/dar[0],
                                        sy[i]/dar[1],
                                        sz[i]/dar[2])
            stream.SetIntegrator(integ)
            stream.Update()
            data = self._cut_data(stream)
            if item.getp('ribbons'):
                streamribbon = vtk.vtkRibbonFilter()
                streamribbon.SetInput(data.GetOutput())
                streamribbon.VaryWidthOn()
                streamribbon.SetWidthFactor(item.getp('ribbonwidth'))
                #streamribbon.SetAngle(90)
                streamribbon.SetDefaultNormal([0,1,0])
                streamribbon.UseDefaultNormalOn()
                streamribbon.Update()
                output = streamribbon.GetOutput()
            elif item.getp('tubes'):
                streamtube = vtk.vtkTubeFilter()
                streamtube.SetInput(data.GetOutput())
                streamtube.SetRadius(1)
                streamtube.SetNumberOfSides(item.getp('n'))
                streamtube.SetVaryRadiusToVaryRadiusByVector()
                streamtube.Update()
                output = streamtube.GetOutput()
            else:
                output = data.GetOutput()
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInput(output)
            mapper.SetLookupTable(self._axis._vtk_colormap)
            caxis = self._axis.getp('caxis')
            if None in caxis:
                caxis = output.GetBounds()[4:]
                #caxis = sgrid.GetScalarRange()
            mapper.SetScalarRange(caxis)
            mapper.Update()
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            #self._set_shading(item, stream, actor)
            self._set_actor_properties(item, actor)
            self._add_legend(item, output)
            self._axis._renderer.AddActor(actor)
            self._axis._vtk_apd.AddInput(output)
            
    def _add_line(self, item):
        dar = self._axis.getp('daspect')
        n = item.getp('numberofpoints')
        polydata = vtk.vtkPolyData()
        polydata.SetLines(vtk.vtkCellArray()) 
        points = vtk.vtkPoints()
        #points.SetNumberOfPoints(n)
        x = ravel(item.getp('xdata'))/dar[0]
        y = ravel(item.getp('ydata'))/dar[1]
        z = item.getp('zdata')
        if z is not None:
            z = ravel(z)/dar[2]
        else:
            z = zeros(n, NumPy_dtype(x))
        ids = vtk.vtkIdList()
        for i in range(1,n):
            points.InsertNextPoint(x[i-1], y[i-1], z[i-1])
            ids.InsertNextId(i-1)
            ids.InsertNextId(i)
            polydata.InsertNextCell(3, ids)
        points.InsertNextPoint(x[n-1], y[n-1], z[n-1]) # last point
        polydata.SetPoints(points)
        polydata.Update()
        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInput(polydata)
        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        self._set_actor_properties(item, actor)
        self._add_legend(item, polydata)
        self._axis._renderer.AddActor(actor)
        self._axis._vtk_apd.AddInput(polydata)

    def _set_actor_properties(self, item, actor):
        # set line properties:
        color = item.getp('linecolor')
        if not isinstance(color, (tuple,list)):
            try: color = self._colors[color]
            except: color = (0,0,1) # use blue as default
        actor.GetProperty().SetColor(color)
        if item.getp('linetype') == '--':
            actor.GetProperty().SetLineStipplePattern(65280)
        elif item.getp('linetype') == ':':
            actor.GetProperty().SetLineStipplePattern(0x1111)
            actor.GetProperty().SetLineStippleRepeatFactor(1)
        #actor.GetProperty().SetPointSize(item.getp('pointsize'))
        linewidth = item.getp('linewidth')
        if linewidth:
            actor.GetProperty().SetLineWidth(float(linewidth))
        
        # set material properties:
        ax = self._axis
        mat = item.getp('material')
        if mat.getp('opacity') is not None:
            actor.GetProperty().SetOpacity(mat.getp('opacity'))
        if mat.getp('ambient') is not None:
            actor.GetProperty().SetAmbient(mat.getp('ambient'))
        if ax.getp('ambientcolor') is not None:
            actor.GetProperty().SetAmbientColor(ax.getp('ambientcolor'))
        if mat.getp('diffuse') is not None:
            actor.GetProperty().SetDiffuse(mat.getp('diffuse'))
        if ax.getp('diffusecolor') is not None:
            actor.GetProperty().SetDiffuseColor(ax.getp('diffusecolor'))
        if mat.getp('specular') is not None:
            actor.GetProperty().SetSpecular(mat.getp('specular'))
        if mat.getp('specularpower') is not None:
            actor.GetProperty().SetSpecularPower(mat.getp('specularpower'))

    def _set_grid(self):
        if not self._axis.getp('visible') or not self._axis.getp('grid'):
            return
        b = self._axis._vtk_box_bounds #self._axis._vtk_scaled_bounds
        if self._axis.getp('camera').getp('view') == 3:
            origins = [[b[0],b[2],b[4]], [b[0],b[3],b[4]], [b[1],b[2],b[4]]]
            points1 = [[b[1],b[2],b[4]], [b[0],b[3],b[5]], [b[1],b[2],b[5]]]
            points2 = [[b[0],b[3],b[4]], [b[1],b[3],b[4]], [b[1],b[3],b[4]]]
        else:
            origins = [[b[0],b[2],0]]
            points1 = [[b[0],b[3],0]]
            points2 = [[b[1],b[2],0]]
        for i in range(len(origins)):
            plane = vtk.vtkPlaneSource()
            plane.SetResolution(4, 4)
            plane.SetOrigin(origins[i])
            plane.SetPoint1(points1[i])
            plane.SetPoint2(points2[i])
            plane.Update()
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInput(plane.GetOutput())
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            actor.GetProperty().SetColor(0,0,0)
            actor.GetProperty().SetRepresentationToWireframe()
            actor.GetProperty().SetLineStipplePattern(0x1111)
            actor.GetProperty().SetLineStippleRepeatFactor(1)
            self._axis._renderer.AddActor(actor)

    def _create_vtk_data(self):
        for item in self._axis.getp('plotitems'):
            func = item.getp('function')
            if isinstance(item, Line):
                self._add_line(item)
            elif isinstance(item, Surface):
                if func == 'pcolor':
                    sgrid = self._get_2d_structured_grid(item, heights=False)
                else:
                    sgrid = self._get_2d_structured_grid(item)
                self._add_surface(item, sgrid)
                citem = item.getp('contours')
                if isinstance(citem, Contours):
                    csgrid = self._get_2d_structured_grid(citem, heights=False,
                                                          bottom=True)
                    self._add_contours(citem, csgrid)
            elif isinstance(item, Contours):
                if item.getp('clocation') == 'surface':
                    sgrid = self._get_2d_structured_grid(item)
                else:
                    sgrid = self._get_2d_structured_grid(item, heights=False)
                self._add_contours(item, sgrid)
            elif isinstance(item, VelocityVectors):
                if len(item.getp('udata').shape) == 3:
                    sgrid = self._get_3d_structured_grid(item, vectors=True)
                else:
                    sgrid = self._get_2d_structured_grid(item, vectors=True)
                self._add_velocity_vectors(item, sgrid)
            elif isinstance(item, Streams):
                if len(item.getp('udata').shape) == 3: 
                    sgrid = self._get_3d_structured_grid(item, vectors=True)
                else:
                    sgrid = self._get_2d_structured_grid(item, vectors=True)
                self._add_streams(item, sgrid)
            elif isinstance(item, Volume):
                sgrid = self._get_3d_structured_grid(item)
                contours = func == 'contourslice'
                if func in ['slice_', 'contourslice']:
                    self._add_slices(item, sgrid, contours=contours)
                elif func == 'isosurface':
                    self._add_isosurface(item, sgrid)
            else:
                raise NotImplementedError('%s not yet implemented' % item)

            self._axis._vtk_apd.Update()

    def _fix_latex(self, legend):
        """Remove latex syntax a la $, \, {, } etc."""
        legend = legend.strip()
        # General fix of latex syntax (more readable)
        legend = legend.replace('**', '^')  
        #legend = legend.replace('*', '')
        legend = legend.replace('$', '')
        legend = legend.replace('{', '')
        legend = legend.replace('}', '')
        legend = legend.replace('\\', '')
        return legend

    def _add_legend(self, item, polydata):
        legend = self._fix_latex(item.getp('legend'))
        if legend:
            ax = self._axis
            n = ax._vtk_nolegends
            ax._vtk_nolegends += 1
            ax._vtk_legendbox.SetNumberOfEntries(ax._vtk_nolegends)
            ax._vtk_legendbox.SetEntrySymbol(n, polydata)
            ax._vtk_legendbox.SetEntryString(n, legend)
            color = self._colors[item.getp('linecolor')]
            ax._vtk_legendbox.SetEntryColor(n, color)#ax.getp('fgcolor'))

    def _set_legends(self):
        ax = self._axis
        n = ax._vtk_nolegends
        if n > 0:
            ax._vtk_legendbox.SetNumberOfEntries(n)
            #ax._vtk_legendbox.ScalarVisibilityOff()
            #ax._vtk_legendbox.BorderOff()
            ax._vtk_legendbox.GetPositionCoordinate().SetValue(0.8, 0.2, 0)
            ax._vtk_legendbox.GetPosition2Coordinate().SetValue(.2, n*.1, 0)
            ax._vtk_legendbox.GetProperty().SetColor(ax.getp('fgcolor'))
            ax._renderer.AddActor(ax._vtk_legendbox)

    def _set_lights(self):
        if not hasattr(self._axis, '_vtk_lights'):
            self._axis._vtk_lights = []
        else: # remove all lights (if any)
            for l in self._axis._vtk_lights:
                self._axis._renderer.RemoveLight(l)
            self._axis._vtk_lights = []
        for l in self._axis.getp('lights'):
            light = vtk.vtkLight()
            light.SetColor(l.getp('lightcolor'))
            light.SetFocalPoint(l.getp('lighttarget'))
            light.SetPosition(l.getp('lightpos'))
            self._axis._renderer.AddLight(light)
            self._axis._vtk_lights.append(light)

    def _setup_axis(self):
        ax = self._axis
        xmin, xmax = ax.getp('xmin'), ax.getp('xmax')
        if None in [xmin, xmax]:
            xmin, xmax = ax.getp('xlim')
        ymin, ymax = ax.getp('ymin'), ax.getp('ymax')
        if None in [ymin, ymax]:
            ymin, ymax = ax.getp('ylim')
        zmin, zmax = ax.getp('zmin'), ax.getp('zmax')
        if None in [zmin, zmax]:
            zmin, zmax = ax.getp('zlim')
        bnds = [xmin, xmax, ymin, ymax, zmin, zmax]
        ax._vtk_bounds = bnds[:]
        # scale axis:
        dar = ax.getp('daspect')
        bnds[0] = bnds[0]/dar[0];  bnds[1] = bnds[1]/dar[0]
        bnds[2] = bnds[2]/dar[1];  bnds[3] = bnds[3]/dar[1]
        bnds[4] = bnds[4]/dar[2];  bnds[5] = bnds[5]/dar[2]
        ax._vtk_scaled_bounds = bnds[:]

##         fig = self.gcf()
##         # clean up:
##         if hasattr(ax, '_renderer'):
##             self._g.RemoveRenderer(ax._renderer)
##             if ax._renderer in fig._renderers:
##                 fig._renderers.remove(ax._renderer)
##             del ax._renderer
        
##         ax._renderer = vtk.vtkRenderer()
##         self._g.AddRenderer(ax._renderer)
##         # add this new renderer to the current figures list of renderers
##         # so we can remove it later (e.g. when using clf()):
##         fig._renderers.append(ax._renderer)

        if not hasattr(ax, '_renderer'):
            ax._renderer = vtk.vtkRenderer()
            self._g.AddRenderer(ax._renderer)
            # add this new renderer to the current figures list of renderers
            # so we can remove it later (e.g. when using clf()):
            gcf()._renderers.append(ax._renderer)
            
        if hasattr(ax, '_vtk_legendbox'):
            ax._renderer.RemoveActor(ax._vtk_legendbox)
        ax._vtk_legendbox = vtk.vtkLegendBoxActor()
        ax._vtk_nolegends = 0
        ax._vtk_legendbox.SetNumberOfEntries(0)

        #ax._renderer.TwoSidedLightingOff()
        ax._renderer.SetBackground(ax.getp('bgcolor'))
        viewport = ax.getp('viewport')
        if not viewport:
            viewport = (0,0,1,1)
        ax._renderer.SetViewport(viewport)
        ax._renderer.RemoveAllViewProps() # clear current scene
        #axshape = self.gcf().getp('axshape')
        #ax._renderer.SetPixelAspect(axshape[1], axshape[0])

        if not hasattr(ax, '_vtk_apd'):
            ax._vtk_apd = vtk.vtkAppendPolyData()
        else:
            ax._vtk_apd.RemoveAllInputs()

    def _replot(self):
        self._axis = gca() # shortcut for fast access
        
        fig = self.gcf()
        if fig.getp('axshape') != fig._axshape:
            # remove all current renderers:
            for ren in fig._renderers:
                self._g.RemoveRenderer(ren)
            fig._renderers = []
            fig._axshape = fig.getp('axshape')
            
        #if self._master is not None:
        #    fig._root.withdraw()

        if len(self._axis.getp('plotitems')) > 0:
            self._setup_axis()
            self._set_lights()
            self._set_colormap()
            self._set_caxis()
            self._create_vtk_data()
            self._create_labeled_axes()
            self._set_view()
            self._set_box_state()
            self._set_colorbar()
            self._set_title()
            self._set_legends()
            self._set_grid()

            # render scene:
            self._axis._renderer.Render()

        if self.getp('show') and hasattr(fig, '_root'):
            fig._root.deiconify() # raise window
            fig._root.update()
            
        # render complete scene:
        self._g.Render()
            
    def figure(self, *args, **kwargs):
        """Extension of BaseClass.figure"""
        fig = BaseClass.figure(self, *args, **kwargs)
        try:
            fig._g
        except:
            try:
                fig._g = self._create_Tk_gui()
            except Tkinter.TclError:
                # can't create gui; only offscreen rendering
                fig._g = vtk.vtkRenderWindow()
                try:
                    width, height = fig.getp('size')
                except TypeError:
                    width, height = (640, 480)
                if width is None or height is None:
                    width, height = (640, 480)
                fig._g.SetSize(width, height)
                fig._g.OffScreenRenderingOn()
            fig._renderers = []
            fig._axshape = fig.getp('axshape')
        self._g = fig._g  # link for faster access
        #self._g.SetAAFrames(5)
        return fig

    def clf(self):
        """Clear current figure."""
        fig = self.gcf()
        for ren in fig._renderers:
            self._g.RemoveRenderer(ren)
        fig._renderers = []
        if self._master is not None:
            fig._root.withdraw() # hide window
        del fig._g
        BaseClass.clf(self)

    def closefig(self, num):
        """Close figure window with number 'num'."""
        if num in self._figs:
            curfig = self._attrs['curfig']
            self.figure(num) # set figure with 'num' as current figure
            self.clf()
            del self._figs[num]
            self.figure(curfig) # put back the current figure
        else:
            print 'no such figure with number', num

    def closefigs(self):
        """Close all figure windows."""
        keys = self._figs.keys()
        for key in keys:
            closefig(key)
        BaseClass.closefigs(self)
        self.figure(self._attrs['curfig'])

    def hardcopy(self, filename="",
                 quality=100,
                 progressive=False,
                 vector_file=True,
                 landscape=False,
                 raster3d=False,
                 compress=False,
                 **kwargs):
        """The figure can be stored in either a vector PostScript (PS/EPS) or
        PDF file using GL2PS or a image file (PNG/PNM/JPEG/TIFF/BMP) using
        a corresponding vtkWriter instance. PostScript output can also be
        generated using vtkPostScriptWriter if vector_file is set to False.
        TeX output is also available, but only the text output will be saved
        to the file.

        If the given filename has no extension, then EPS output will be used.

        Keyword arguments:

          quality     -- Sets the quality of the resulting image. Affects only
                         JPEG images. Given as an integer between 0 and 100
                         where 100 results in the best quality (but also
                         the largest file). The default is quality=100.

          progressive -- Sets whether to use progressive JPEG generation or
                         not. Defaults to False.

          vector_file -- If True (default), the figure will be stored as a
                         vector file. This is only true if either PS, EPS,
                         or PDF are choosen as the output file format.
          
        Additional keyword arguments (only in affect if the vector_file
        argument is set to True and the file format is either PS, EPS or PDF):

          landscape -- Sets whether to use landscape or portrait orientation.
                       A True value result in landscape orientation. Defaults
                       to False.
          
          raster3d  -- If True, this will write 3D props as raster images
                       while 2D props are rendered using vector graphic
                       primitives. Defaults to False.
          
          compress  -- Compression when generating PostScript or PDF output.
                       The default is False (no compression).
        """
        if not filename:
            raise TypeError("hardcopy: No filename given, cannot save figure.")

        self.setp(**kwargs)

        basename, ext = os.path.splitext(filename)
        if not ext:  # no extension given, use .eps
            ext = '.eps'
            filename += ext
        
        if not self.getp('show'):  # don't display to screen
            self._g.OffScreenRenderingOn()

        replot = kwargs.get('replot', True)
        if replot:
            self._replot()

        color = self.getp('color')
        
        vector_file_formats = {'.ps': 0, '.eps': 1, '.pdf': 2, '.tex': 3}
        if vector_file and ext.lower() in vector_file_formats:
            exp = vtk.vtkGL2PSExporter()
            exp.SetRenderWindow(self._g)
            exp.SetFilePrefix(basename)
            exp.SetFileFormat(vector_file_formats[ext.lower()])
            exp.SetCompress(bool(compress))
            exp.SetLandscape(bool(landscape))
            exp.SetSortToBSP()
            #exp.SetSortToSimple() # less expensive sort algorithm
            exp.DrawBackgroundOn()
            exp.SetWrite3DPropsAsRasterImage(bool(raster3d))
            exp.Write()
        else:
            image_writers = {
                '.tif': vtk.vtkTIFFWriter(),
                '.bmp': vtk.vtkBMPWriter(),
                '.pnm': vtk.vtkPNMWriter(),
                '.png': vtk.vtkPNGWriter(),
                '.jpg': vtk.vtkJPEGWriter(),
                '.ps': vtk.vtkPostScriptWriter(),
                '.eps': vtk.vtkPostScriptWriter(), # no EPS file
                }
            w2if = vtk.vtkWindowToImageFilter()
            w2if.SetInput(self._g)
            try:
                writer = image_writers[ext.lower()]
            except KeyError:
                raise TypeError(
                    "hardcopy: File format '%s' is currently not supported."\
                    % ext)
            try:
                writer.SetQuality(int(quality))
                writer.SetProgressive(bool(progressive))
            except ValueError:
                raise ValueError(\
                      "hardcopy: Integer required for the 'quality' argument.")
            except AttributeError:
                pass # only vtkJPEGWriter has quality and progressive attrs.
            writer.SetFileName(filename)
            writer.SetInput(w2if.GetOutput())
            writer.Write()
        self._g.OffScreenRenderingOff()

    hardcopy.__doc__ = BaseClass.hardcopy.__doc__ + hardcopy.__doc__

    def brighten(self, *args):
        """Brighten or darken color map."""
        nargs = len(args)
        if nargs == 2: # brighten(map,beta)
            if not isinstance(args[0], vtk.vtkLookupTable):
                raise ValueError("brighten: map must be %s, not %s" % \
                                 (type(vtk.vtkLookupTable), type(args[0])))
            lut, beta = args
        elif nargs == 1: # brighten(beta)
            if not hasattr(self._axis, '_vtk_colormap'):
                print "brighten: no colormap set."
                return
            lut = self._axis._vtk_colormap
            beta = args[0]
        if not isinstance(beta, (float,int)) or (beta <= -1 or beta >= 1):
            raise ValueError("brighten: beta must be between -1 and 1")
        # all is OK, change colormap:
        hue = lut.GetHueRange()
        val = lut.GetValueRange()
 
    # colormaps
    def hsv(self, m=256):
        lut = vtk.vtkLookupTable()
        lut.SetHueRange(0.0, 1.0)
        lut.SetSaturationRange(1.0, 1.0)
        lut.SetValueRange(1.0, 1.0)
        lut.SetNumberOfColors(m)
        lut.Build()
        return lut

    def gray(self, m=256):
        lut = vtk.vtkLookupTable()
        lut.SetHueRange(0.0, 0.0)
        lut.SetSaturationRange(0.0, 0.0)
        lut.SetValueRange(0.0, 1.0)
        lut.SetNumberOfColors(m)
        lut.Build()
        return lut

    def hot(self, m=256):
        lut = vtk.vtkLookupTable()
        inc = 0.01175
        lut.SetNumberOfColors(256)
        i = 0
        r = 0.0; g = 0.0; b = 0.0
        while r <= 1.:
            lut.SetTableValue(i, r, g, b, 1)
            r += inc;  i += 1
        r = 1.
        while g <= 1.:
            lut.SetTableValue(i, r, g, b, 1)
            g += inc;  i += 1
        g = 1.
        while b <= 1:
            if i == 256: break
            lut.SetTableValue(i, r, g, b, 1)
            b += inc;  i += 1
        lut.Build()
        return lut

    def flag(self, m=64):
        """Alternating red, white, blue, and black color map.

        - flag(m)
          'm' must be a multiple of 4
        """
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        # the last parameter alpha is set to 1 by default
        # in method declaration
        for i in range(0,m,4):
            lut.SetTableValue(i,1,0,0,1)   # red
            lut.SetTableValue(1+i,1,1,1,1) # white
            lut.SetTableValue(2+i,0,0,1,1) # blue
            lut.SetTableValue(3+i,0,0,0,1) # black
        lut.Build()
        return lut

    def jet(self, m=256):
        # blue, cyan, green, yellow, red, black
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        lut.SetHueRange(0.667,0.0)
        lut.Build()
        return lut

    def blue_to_yellow(self, m=200):
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        for i in range(m):
            frac = i / float(m / 2.0 - 1.0)
            if (frac <= 1):
                r = frac
                g = r
                b = 1
            else:
                r = 1
                g = r
                b = 2 - frac
            # SetTableValue(indx, red, green, blue, alpha)
            lut.SetTableValue(i, r, g, b, 1)
        lut.Build()
        return lut

    def spring(self, m=256):
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        lut.SetHueRange(0.0, 0.17)
        lut.SetSaturationRange(0.5, 1.0) 
        lut.SetValueRange(1.0, 1.0)
        lut.Build()
        return lut

    def summer(self, m=256):
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        lut.SetHueRange(0.47, 0.17)
        lut.SetSaturationRange(1.0, 0.6) 
        lut.SetValueRange(0.5, 1.0)
        lut.Build()
        return lut

    def winter(self, m=256):
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        lut.SetHueRange(0.8, 0.42)
        lut.SetSaturationRange(1.0, 1.0) 
        lut.SetValueRange(0.6, 1.0)
        lut.Build()
        return lut
        
    def autumn(self, m=256):
        lut = vtk.vtkLookupTable()
        lut.SetNumberOfColors(m)
        lut.SetHueRange(0.0, 0.15)
        lut.SetSaturationRange(1.0, 1.0) 
        lut.SetValueRange(1.0, 1.0)
        lut.Build()
        return lut
    
plt = VtkBackend() # Create backend instance
use(plt, globals()) # Export public namespace of plt to globals()
backend = os.path.splitext(os.path.basename(__file__))[0][:-1]




from scitools.numpyutils import zeros, ones, exp, reshape, ravel, \
     ndgrid, seq, linspace, arctan2, sqrt, shape, log, sin, cos

def available_backends():
    """Return a list of the available backends."""
    import os
    from scitools.misc import test_if_module_exists
    files = os.listdir(os.path.dirname(__file__))
    files.remove('template_.py')
    if '__init__.py' in files:
        files.remove('__init__.py')
    backends = [f for f in files if f.endswith('_.py')]
    available = []
    print '\nChecking if you have all the Easyviz backends...'
    for b in backends:
        module = 'scitools.easyviz.' + b[:-3]
        try:
            test_if_module_exists(module, abort=False, raise_exception=True, msg='')
            available.append(module[17:])
        except:
            pass
            #print "You can't use the %s backend" % module
    return available
            
    
def peaks(*args):
    # z = peaks()
    # z = peaks(n)
    # z = peaks(x,y)
    n = 49
    nargs = len(args)
    if nargs in (0,1):
        if nargs == 1:
            n = int(args[0])
        x, y = ndgrid(linspace(-3,3,n),linspace(-3,3,n))
    elif nargs == 2:
        x, y = args
    else:
        raise SyntaxError("Invalid number of arguments.")
    return 3*(1-x)**2*exp(-x**2-(y+1)**2) \
           - 10*(x/5-x**3-y**5)*exp(-x**2-y**2) - 1/3*exp(-(x+1)**2-y**2)

def flow(*args):
    # xx,yy,zz,vv = flow()
    # xx,yy,zz,vv = flow(n)
    # xx,yy,zz,vv = flow(xx,yy,zz)
    if len(args) == 0:
        xx, yy, zz = ndgrid(linspace(0.1, 10, 50),
                            linspace(-3, 3, 25),
                            linspace(-3, 3, 25),
                            sparse=False)
    elif len(args) == 1:
        n = int(args[0])
        xx, yy, zz = ndgrid(linspace(0.1, 10, 2*n),
                            linspace(-3, 3, n),
                            linspace(-3, 3, n),
                            sparse=False)
    elif len(args) == 3:
        xx, yy, zz = args
    else:
        raise SyntaxError("Invalid number of arguments.")
    
    # convert to spherical coordinates:
    theta = arctan2(zz, yy)
    phi = arctan2(xx, sqrt(yy**2 + zz**2))
    r = sqrt(xx**2 + yy**2 + zz**2)

    rv = 2/r*(3/(2-cos(phi))**2 - 1)
    phiv = -2*sin(phi)/(2-cos(phi))/r
    thetav = zeros(shape(r))

    # convert back to cartesian coordinates:
    xv = rv*cos(phiv)*cos(thetav)
    yv = rv*cos(phiv)*sin(thetav)
    zv = rv*sin(phiv)

    vv = log(sqrt(xv**2 + yv**2 + zv**2))

    return xx, yy, zz, vv



#!/usr/bin/env python

"""Demonstration on how to place arbitrary axes in the different backends."""

from scitools.std import *

x = linspace(0,1,51)
y1 = sin(2*pi*x)
y2 = cos(4*pi*x)

plot(x, y1, 'rd-', xlabel='x-axis', ylabel='y-axis', legend='test')

if backend == 'veusz':
    pos = [.64, .55, .1, .2]  # [leftMargin,bottomMargin,rightMargin,topMargin]
elif backend == 'gnuplot':
    pos = [.6, .52, .33, .33] # [left,bottom,width,height]
elif backend == 'grace':
    pos = [.6, .5, .8, .75]   # [xmin,ymin,xmax,ymax]
elif backend == 'matplotlib':
    pos = [.62, .56, .2, .2]  # [left,bottom,width,height]
elif backend == 'matlab':
    pos = [.62, .56, .2, .2]  # [left,bottom,width,height]
elif backend == 'matlab2':
    pos = [.62, .56, .2, .2]  # [left,bottom,width,height]
elif backend == 'pyx':
    # default figure size (i.e., [width,height]) is [15,9.27].
    pos = [9.6, 3.5, 4, 4]    # [xpos,ypos,width,height]
else:
    print "The '%s' backend has currently no support for placement of " \
          "arbitrary axes." % backend
    pos = None

if pos is not None:
    ax = axes(viewport=pos)  # create a new axis at the given position
    plot(ax, x, y2, 'b--')   # draw a curve in the new axis

hardcopy('tmp_axes1a.eps')
if not backend == 'pyx':
    hardcopy('tmp_axes1a.png')

if backend == 'matlab2':
    save('tmp_test_axes.m')
elif backend == 'veusz':
    save('tmp_test_axes.vsz')

raw_input('Press Return key to quit:')



from scitools.std import *   # for curve plotting

def f1(t):
    return t**2*exp(-t**2)

def f2(t):
    return t**2*f1(t)

t = linspace(0, 3, 51)
y1 = f1(t)
y2 = f2(t)

plot(t, y1, 'r-', t, y2, 'bo',
     xlabel='t', ylabel='y',
     axis=[0, 4, -0.1, 0.6],
     legend=('t^2*exp(-t^2)', 't^4*exp(-t^2)'),
     title='Plotting two curves in the same plot',
     savefig='tmp2.eps')

# grab the backend plotting object and work with the plotting
# program directly:

if backend == 'gnuplot':
    g = get_backend()
    #g.__class__.__name__ == 'Gnuplot'
    # g is a Gnuplot object, work with Gnuplot commands directly:
    g('set label "global maximum" at 0.1,0.5 font "Times,18"')
    g('set arrow from 0.5,0.48 to 0.98,0.37 linewidth 2')
    g('save "tmp.gnu"')  # save all settings for later tuning
    g.refresh()
    g.hardcopy('tmp2.eps')  # make new hardcopy (do not use savefig/hardcopy)

fig = gcf(); print fig.dump()

raw_input('Press Return key to quit: ')



#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/visualize/f5-7858.html

from scitools.easyviz import *
from time import sleep
from scipy import io

setp(interactive=False)

# Create an Isosurface:
wind = io.loadmat('wind_matlab_v6.mat')
x = wind['x']
y = wind['y']
z = wind['z']
u = wind['u']
v = wind['v']
w = wind['w']

sx, sy, sz = ndgrid([80]*36,seq(20,55,1),[5]*36, sparse=True)

wind_speed = sqrt(u**2 + v**2 + w**2)

hiso = isosurface(x,y,z,wind_speed,40,indexing='xy')
#isonormals(x,y,z,wind_speed,hiso)
#set(hiso,'FaceColor','red','EdgeColor','none');
hold('on')
shading('interp')

# Add Isocaps to the Isosurface:
#hcap = patch(isocaps(x,y,z,wind_speed,40),...
#    'FaceColor','interp',...
#    'EdgeColor','none');
try:
    colormap(hsv())
except:
    pass

# Create First Set of Cones:
daspect([1,1,1])
#[f verts] = reducepatch(isosurface(x,y,z,wind_speed,30),0.07);
isosurface(x,y,z,wind_speed,30,indexing='xy')
#h1 = coneplot(x,y,z,u,v,w,verts(:,1),verts(:,2),verts(:,3),3);
#set(h1,'FaceColor','blue','EdgeColor','none');

# Create Second Set of Cones:
#xrange = linspace(min(x(:)),max(x(:)),10);
#yrange = linspace(min(y(:)),max(y(:)),10);
#zrange = 3:4:15;
#[cx,cy,cz] = ndgrid(xrange,yrange,zrange);
#h2 = coneplot(x,y,z,u,v,w,cx,cy,cz,2);
#set(h2,'FaceColor','green','EdgeColor','none');

# Define the View:
axis('tight')
box('on')
camproj('perspective')
camzoom(1.25)
view(65,45)

# Add Lighting:
#camlight(-45,45)
#set(gcf,'Renderer','zbuffer');
#lighting phong
#set(hcap,'AmbientStrength',.6)

show()

raw_input('Press Return key to quit: ')



"""Test functions for <template> command."""

from easyviztest import *


class test_box_basic(EasyvizTestCase):
    def check_box_on(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        box('on')
        title("surf(..);box('on')")
        n()

    def check_box_on_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,box='on')
        title("surf(..,box='on')")
        n()

    def check_box_off(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        box('on')
        box('off')
        title("surf(..);box('on');box('off')")
        n()

    def check_box_toggle_on(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        box()
        title("surf(..);box()")
        n()

    def check_box_toggle_off(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        box()
        box()
        title("surf(..);box();box()")
        n()

    def check_box_on_specify_axis(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        box(gca(),'on')
        title("surf(..);box(gca(),'on')")
        n()

    def check_box_toggle_on_specify_axis(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        box(gca())
        title("surf(..);box(gca())")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_box_basic,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""
This backend is based on the scientific plotting package Veusz. One can use
this backend by

  python somefile.py --SCITOOLS_easyviz_backend veusz

or one can specify the backend in the SciTools configuration file
scitools.cfg under the [easyviz] section

  [easyviz]
  backend = veusz

and then

  from scitools.std import *

or if just easyviz is needed

  from scitools.easyviz import *

REQUIREMENTS:

Veusz 0.10

TODO:

- update to version 0.99 of Veusz

Known issues:

- If an image of a 2D dataset (for instance created by pcolor) is drawn before
  a curve or contour lines, then these will be completely hidden by the image.
  This can be fixed by moving the image down the tree in the Veusz gui.

- savefig does not work

Tip:

- Here is an example on how to create axes at arbitrary positions in the
  Veusz backend:

    >>> x = linspace(0,1,51)
    >>> y1 = sin(2*pi*x)
    >>> y2 = cos(4*pi*x)
    >>> plot(x, y1, 'rd-')
    [<scitools.easyviz.common.Line object at 0xb65694ac>]
    >>> hold('on')
    >>> ax = axes(viewport=['9cm','9cm','1cm','2cm'])
    >>> plot(ax, x, y2, 'b--')
    [<scitools.easyviz.common.Line object at 0xb2c36d0c>]
    >>> 

  The optional argument viewport to the axes command must specify a list with
  four elements [left,bottom,right,top] where each of the elements defines
  the margins around the axes.

"""

from __future__ import division

from common import *
from scitools.globaldata import DEBUG, VERBOSE
from scitools.misc import test_if_module_exists 

test_if_module_exists('veusz', msg='You need to install the Veusz package.', abort=False)

import veusz.embed
import os


class VeuszBackend(BaseClass):
    def __init__(self):
        BaseClass.__init__(self)
        self._init()
        
    def _init(self, *args, **kwargs):
        """Perform initialization that is special for this backend."""
        
        # Set docstrings of all functions to the docstrings of BaseClass
        # The exception is if something is very different
        

        # Calling figure method here makes the program halt.
        #self.figure(self.getp('curfig'))

        # conversion tables for format strings:
        self._markers = {
            '': None,            # no marker
            '.': 'dot',          # dot
            'o': 'circle',       # circle
            'x': 'linecross',    # cross
            '+': 'lineplus',     # plus sign
            '*': 'asterisk',     # asterisk
            's': 'square',       # square
            'd': 'diamond',      # diamond
            '^': 'triangle',     # triangle (up)
            'v': 'triangledown', # triangle (down) 
            '<': 'triangle',     # triangle (left) --> up
            '>': 'triangledown', # triangle (right) --> down
            'p': 'star',         # pentagram
            'h': 'cross',        # hexagram --> filled cross
            }
                                 
        self._colors = {
            '': None,       # no color --> blue
            'r': 'red',     # red
            'g': 'green',   # green
            'b': 'blue',    # blue
            'c': 'cyan',    # cyan
            'm': 'magenta', # magenta
            'y': 'yellow',  # yellow
            'k': 'black',   # black
            'w': 'white',   # white
            }

        self._line_styles = {
            '': None,         # no line 
            '-': 'solid',     # solid line
            ':': 'dotted',    # dotted line
            '-.': 'dash-dot', # dash-dot line
            '--': 'dashed',   # dashed line
            }

        # convert table for colorbar location:
        self._colorbar_locations = {
            'North': None,
            'South': None,
            'East': None,
            'West': None,
            'NorthOutside': None,
            'SouthOutside': None,
            'EastOutside': None,
            'WestOutside': None,
            }

        if DEBUG:
            print "Setting backend standard variables"
            for disp in 'self._markers self._colors self._line_styles'.split():
                print disp, eval(disp)

    def _set_scale(self, ax):
        """Set linear or logarithmic (base 10) axis scale."""
        if DEBUG:
            print "Setting scales"
        scale = ax.getp('scale')
        if scale == 'loglog':
            # use logarithmic scale on both x- and y-axis
            self._g.Set('x/log', True)
            self._g.Set('y/log', True)
        elif scale == 'logx':
            # use logarithmic scale on x-axis and linear scale on y-axis
            self._g.Set('x/log', True)
            self._g.Set('y/log', False)
        elif scale == 'logy':
            # use linear scale on x-axis and logarithmic scale on y-axis
            self._g.Set('x/log', False)
            self._g.Set('y/log', True)
        elif scale == 'linear':
            # use linear scale on both x- and y-axis
            self._g.Set('x/log', False)
            self._g.Set('y/log', False)

    def _set_labels(self, ax):
        """Add text labels for x-, y-, and z-axis."""
        if DEBUG:
            print "Setting labels"
        xlabel = ax.getp('xlabel')
        ylabel = ax.getp('ylabel')
        zlabel = ax.getp('zlabel')
        fontsize = ax.getp('fontsize')
        if xlabel:
            # add a text label on x-axis
            self._g.Set('x/label', xlabel)
            self._g.Set('x/Label/size', str(fontsize))
        if ylabel:
            # add a text label on y-axis
            self._g.Set('y/label', ylabel)
            self._g.Set('y/Label/size', str(fontsize))
        if zlabel:
            # add a text label on z-axis
            pass
        
    def _set_title(self, ax):
        """Add a title at the top of the axis."""
        if DEBUG:
            print "Setting title"
        title = self._fix_latex(ax.getp('title'))
        if title:
            # set title
            label = self._g.Add('label')
            self._g.Set('%s/label' % label, title)
            self._g.Set('%s/xPos' % label, 0.1)
            self._g.Set('%s/yPos' % label, 1.01)
    
    def _set_limits(self, ax):
        """Set axis limits in x, y, and z direction."""
        if DEBUG:
            print "Setting axis limits"
        mode = ax.getp('mode')
        if mode == 'auto':
            # let plotting package set 'nice' axis limits in the x, y,
            # and z direction. If this is not automated in the plotting
            # package, one can use the following limits:
            #xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            pass
        elif mode == 'manual':
            # (some) axis limits are frozen
            xmin = ax.getp('xmin')
            xmax = ax.getp('xmax')
            if xmin is not None and xmax is not None:
                # set x-axis limits
                self._g.Set('x/min', xmin)
                self._g.Set('x/max', xmax)
            else:
                # let plotting package set x-axis limits or use
                #xmin, xmax = ax.getp('xlim')
                pass

            ymin = ax.getp('ymin')
            ymax = ax.getp('ymax')
            if ymin is not None and ymax is not None:
                # set y-axis limits
                self._g.Set('y/min', ymin)
                self._g.Set('y/max', ymax)
            else:
                # let plotting package set y-axis limits or use
                #ymin, ymax = ax.getp('ylim')
                pass

            zmin = ax.getp('zmin')
            zmax = ax.getp('zmax')
            if zmin is not None and zmax is not None:
                # set z-axis limits
                pass
            else:
                # let plotting package set z-axis limits or use
                #zmin, zmax = ax.getp('zlim')
                pass
        elif mode == 'tight':
            # set the limits on the axis to the range of the data. If
            # this is not automated in the plotting package, one can
            # use the following limits:
            xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            self._g.Set('x/min', xmin)
            self._g.Set('x/max', xmax)
            self._g.Set('y/min', ymin)
            self._g.Set('y/max', ymax)
        elif mode == 'fill':
            # not sure about this
            pass

    def _set_position(self, ax):
        """Set axes position."""
        rect = ax.getp('viewport')
        if not ax.getp('pth'):
            if rect:
                # axes position is defined. In Matlab rect is defined as
                # [left,bottom,width,height], where the four parameters are
                # location values between 0 and 1 ((0,0) is the lower-left
                # corner and (1,1) is the upper-right corner).
                # NOTE: This can be different in the plotting package.
                # In Veusz, rect should be defined as [left,bottom,right,top]
                # left, bottom, right, and top are strings defining the
                # graph margins.
                left, bottom, right, top = rect
                self._g.Set('leftMargin', str(left))
                self._g.Set('rightMargin', str(right))
                self._g.Set('topMargin', str(top))
                self._g.Set('bottomMargin', str(bottom))
            else:
                self._g.Set('leftMargin', '1.5cm')
                self._g.Set('rightMargin', '0.1cm')
                self._g.Set('topMargin', '0.9cm')
                self._g.Set('bottomMargin', '1.5cm')

    def _set_daspect(self, ax):
        """Set data aspect ratio."""
        if ax.getp('daspectmode') == 'manual':
            dar = ax.getp('daspect')  # dar is a list (len(dar) is 3).
            pass
        else:
            # daspectmode is 'auto'. Plotting package handles data
            # aspect ratio automatically.
            pass
        
    def _set_axis_method(self, ax):
        method = ax.getp('method')
        if method == 'equal':
            # tick mark increments on the x-, y-, and z-axis should
            # be equal in size.
            pass
        elif method == 'image':
            # same effect as axis('equal') and axis('tight')
            pass
        elif method == 'square':
            # make the axis box square in size
            pass
        elif method == 'normal':
            # full size axis box
            pass
        elif method == 'vis3d':
            # freeze data aspect ratio when rotating 3D objects
            pass

    def _set_coordinate_system(self, ax):
        """
        Use either the default Cartesian coordinate system or a
        matrix coordinate system.
        """
        
        direction = ax.getp('direction')
        if direction == 'ij':
            # Use matrix coordinates. The origin of the coordinate
            # system is the upper-left corner. The i-axis should be
            # vertical and numbered from top to bottom, while the j-axis
            # should be horizontal and numbered from left to right.
            pass
        elif direction == 'xy':
            # use the default Cartesian axes form. The origin is at the
            # lower-left corner. The x-axis is vertical and numbered
            # from left to right, while the y-axis is vertical and
            # numbered from bottom to top.
            pass

    def _set_box(self, ax):
        """Turn box around axes boundary on or off."""
        if DEBUG:
            print "Setting box"
        if ax.getp('box'):
            # display box 
            pass
        else:
            # do not display box
            pass
        
    def _set_grid(self, ax):
        """Turn grid lines on or off."""
        if DEBUG:
            print "Setting grid"
        if ax.getp('grid'):
            # turn grid lines on
            self._g.Set('x/GridLines/hide', False)
            self._g.Set('y/GridLines/hide', False)
        else:
            # turn grid lines off
            self._g.Set('x/GridLines/hide', True)
            self._g.Set('y/GridLines/hide', True)

    def _set_hidden_line_removal(self, ax):
        """Turn on/off hidden line removal for meshes."""
        if DEBUG:
            print "Setting hidden line removal"
        if ax.getp('hidden'):
            # turn hidden line removal on
            pass
        else:
            # turn hidden line removal off
            pass

    def _set_colorbar(self, ax):
        """Add a colorbar to the axis."""
        if DEBUG:
            print "Setting colorbar"
        cbar = ax.getp('colorbar')
        if cbar.getp('visible'):
            # turn on colorbar
            cbar_title = cbar.getp('cbtitle')
            cbar_location = self._colorbar_locations[cbar.getp('cblocation')]
            # ...
        else:
            # turn off colorbar
            pass

    def _set_caxis(self, ax):
        """Set the color axis scale."""
        if DEBUG:
            print "Setting caxis"
        if ax.getp('caxismode') == 'manual':
            cmin, cmax = ax.getp('caxis')
            # NOTE: cmin and cmax might be None:
            if cmin is None or cmax is None:
                cmin, cmax = [0,1]
            # set color axis scaling according to cmin and cmax
            pass
        else:
            # use autoranging for color axis scale
            pass

    def _set_colormap(self, ax):
        """Set the colormap."""
        if DEBUG:
            print "Setting colormap"
        cmap = ax.getp('colormap')
        # cmap is plotting package dependent

    def _set_view(self, ax):
        """Set viewpoint specification."""
        if DEBUG:
            print "Setting view"
        cam = ax.getp('camera')
        view = cam.getp('view')
        if view == 2:
            # setup a default 2D view
            pass
        elif view == 3:
            az = cam.getp('azimuth')
            el = cam.getp('elevation')
            if az is None or el is None:
                # azimuth or elevation is not given. Set up a default
                # 3D view (az=-37.5 and el=30 is the default 3D view in
                # Matlab).
                pass
            else:
                # set a 3D view according to az and el
                pass
            
            if cam.getp('cammode') == 'manual':
                # for advanced camera handling:
                roll = cam.getp('camroll')
                zoom = cam.getp('camzoom')
                dolly = cam.getp('camdolly')
                target = cam.getp('camtarget')
                position = cam.getp('campos')
                up_vector = cam.getp('camup')
                view_angle = cam.getp('camva')
                projection = cam.getp('camproj')

    def _set_axis_props(self, ax):
        if DEBUG:
            print "Setting axis properties"
        self._set_title(ax)
        self._set_scale(ax)
        self._set_limits(ax)
        self._set_position(ax)
        self._set_axis_method(ax)
        self._set_daspect(ax)
        self._set_coordinate_system(ax)
        self._set_hidden_line_removal(ax)
        self._set_colorbar(ax)
        self._set_caxis(ax)
        self._set_colormap(ax)
        self._set_view(ax)
        if ax.getp('visible'):
            self._set_labels(ax)
            self._set_box(ax)
            self._set_grid(ax)
            fontsize = ax.getp('fontsize')
            self._g.Set('x/TickLabels/size', '%spt' % fontsize)
            self._g.Set('y/TickLabels/size', '%spt' % fontsize)
        else:
            # turn off all axis labeling, tickmarks, and background
            self._g.Set('x/Line/hide', True)
            self._g.Set('y/Line/hide', True)
            self._g.Set('x/MajorTicks/hide', True)
            self._g.Set('y/MajorTicks/hide', True)
            self._g.Set('x/MinorTicks/hide', True)
            self._g.Set('y/MinorTicks/hide', True)
            self._g.Set('x/TickLabels/hide', True)
            self._g.Set('y/TickLabels/hide', True)
            self._g.Set('Border/hide', True)
        # set background color off:
        self._g.Set('Background/hide', True)

    def _get_linespecs(self, item):
        """
        Return the line marker, line color, line style, and
        line width of the item.
        """
        
        marker = self._markers[item.getp('linemarker')]
        color = self._colors[item.getp('linecolor')]
        style = self._line_styles[item.getp('linetype')]
        width = item.getp('linewidth')
        return marker, color, style, width

    def _fix_latex(self, legend):
        """
        Veusz understands a limited set of LaTeX-like formatting for
        text. There are some differences (for example,"10^23" puts the
        2 and 3 into superscript), but it is fairly similar. You should
        also leave out the dollar signs. Veusz supports superscripts ("^"),
        subscripts ("_"), brackets for grouping attributes are "{" and "}".

        Supported LaTeX symbols include: \AA, \Alpha, \Beta, \Chi, \Delta,
        \Epsilon, \Eta, \Gamma, \Iota, \Kappa, \Lambda, \Mu, \Nu, \Omega,
        \Omicron, \Phi, \Pi, \Psi, \Rho, \Sigma, \Tau, \Theta, \Upsilon, \Xi,
        \Zeta, \alpha, \approx, \ast, \asymp, \beta, \bowtie, \bullet, \cap,
        \chi, \circ, \cup, \dagger, \dashv, \ddagger, \deg, \delta, \diamond,
        \divide, \doteq, \downarrow, \epsilon, \equiv, \eta, \gamma, \ge, \gg,
        \in, \infty, \int, \iota, \kappa, \lambda, \le, \leftarrow, \lhd, \ll,
        \models, \mp, \mu, \neq, \ni, \nu, \odot, \omega, \omicron, \ominus,
        \oplus, \oslash, \otimes, \parallel, \perp, \phi, \pi, \pm, \prec,
        \preceq, \propto, \psi, \rhd, \rho, \rightarrow, \sigma, \sim, \simeq,
        \sqrt, \sqsubset, \sqsubseteq, \sqsupset, \sqsupseteq, \star, \stigma,
        \subset, \subseteq, \succ, \succeq, \supset, \supseteq, \tau, \theta,
        \times, \umid, \unlhd, \unrhd, \uparrow, \uplus, \upsilon, \vdash,
        \vee, \wedge, xi, \zeta. Please request additional characters if they
        are required (and exist in the unicode character set). Special symbols
        can be included directly from a character map.

        Other LaTeX commands are supported. "\\" breaks a line.
        This can be used for simple tables. For example "{a\\b} {c\\d}"
        shows "a c" over "b d". The command "\frac{a}{b}" shows a
        vertical fraction a/b.
        """
        legend = legend.strip()
        if '^' in legend:
            print '''\
...warning regarding veusz syntax:
   use {} around the arguments in power expressions with ^ (hat)
'''
        # General fix of latex syntax (more readable)
        #legend = legend.replace('**', '^')  
        #legend = legend.replace('*', '')
        legend = legend.replace('$', '')
        #legend = legend.replace('\\', '')
        # fix sin, cos, exp, ln, log, etc:
        def _fix_func(func, newfunc, legend):
            if ('\\'+func) in  legend:
                legend = legend.replace('\\'+func, newfunc)
            return legend
        funcs = 'sin', 'cos', 'tan', 'exp', 'ln', 'log', \
                'sinh', 'cosh', 'tanh'
        for func in funcs:
            legend = _fix_func(func, func, legend)
        funcs = [('atan', 'arctan'), ('asin', 'arcsin'), 
                 ('acos', 'arccos'),]
        for func, newfunc in funcs:
            legend = _fix_func(func, newfunc, legend)

        return legend

    def _add_line(self, item, name):
        """Add a 2D or 3D curve to the scene."""
        if DEBUG:
            print "Adding a line"
        # get data:
        x = item.getp('xdata')
        y = item.getp('ydata')
        z = item.getp('zdata')
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)

        xy = self._g.Add('xy')
        if marker:
            self._g.Set('%s/marker' % xy, marker)
            self._g.Set('%s/MarkerLine/color' % xy, color)
            self._g.Set('%s/MarkerFill/hide' % xy, True)
            if not style:
                self._g.Set('%s/PlotLine/hide' % xy, True)
            else:
                self._g.Set('%s/PlotLine/style' % xy, style)
        else:
            self._g.Set('%s/marker' % xy, 'none')
            if not style:
                style = 'solid'
            self._g.Set('%s/PlotLine/style' % xy, style)
        self._g.Set('%s/PlotLine/color' % xy, color)
        if not width:
            width = 0.5
        self._g.Set('%s/PlotLine/width' % xy, str(width)+'pt')
        
        if z is not None:
            # zdata is given, add a 3D curve:
            pass
        else:
            # no zdata, add a 2D curve:
            self._g.SetData('x%s' % name, x)
            self._g.SetData('y%s' % name, y)
            self._g.Set('%s/xData' % xy, 'x%s' % name)
            self._g.Set('%s/yData' % xy, 'y%s' % name)

        legend = self._fix_latex(item.getp('legend'))
        if legend:
            self._g.Set('%s/key' % xy, legend)

    def _add_surface(self, item, name, shading='faceted', cmap=None):
        if DEBUG:
            print "Adding a surface"
        x = item.getp('xdata')  # grid component in x-direction
        y = item.getp('ydata')  # grid component in y-direction
        z = item.getp('zdata')  # scalar field
        c = item.getp('cdata')  # pseudocolor data (can be None)
        
        img = self._g.Add('image')
                    
        contours = item.getp('contours')
        if contours:
            # the current item is produced by meshc or surfc and we
            # should therefore add contours at the bottom:
            #self._add_contours(contours, placement='bottom')
            pass

        self._g.SetData2D('values%s' % name, z)

        if item.getp('wireframe'):
            # wireframe mesh (as produced by mesh or meshc)
            pass
        else:
            # colored surface (as produced by surf, surfc, or pcolor)
            # use keyword argument shading to set the color shading mode
            self._g.Set('%s/data' % img, 'values%s' % name)
        if cmap is None:
            cmap = 'spectrum'
        self._g.Set('%s/colorMap' % img, cmap)
        
        legend = self._fix_latex(item.getp('legend'))
        if legend:
            self._g.Set('%s/key' % img, legend)

    def _add_contours(self, item, name, placement=None):
        # The placement keyword can be either None or 'bottom'. The
        # latter specifies that the contours should be placed at the
        # bottom (as in meshc or surfc).
        if DEBUG:
            print "Adding contours"
        x = item.getp('xdata')  # grid component in x-direction
        y = item.getp('ydata')  # grid component in y-direction
        z = item.getp('zdata')  # scalar field
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)

        filled = item.getp('filled')  # draw filled contour plot if True
        
        cntr = self._g.Add('contour')

        # set line properties:
        if not width:
            width = 0.5
        width = str(width) + 'pt'
        if not style:
            style = 'solid'
        if not color:
            color = 'black'
        self._g.Set('%s/lines' % cntr, [(style, width, color, False)])

        if filled:
            self._g.Set('%s/fills' % cntr,
                        [('solid', color, False), ('solid', 'white', False)])

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            self._g.Set('%s/numLevels' % cntr, clevels)
        else:
            #self._g.Set('%s/manualLevels' % cntr, cvector)
            # setting manual levels doesn't seem to work
            self._g.Set('%s/numLevels' % cntr, len(cvector))

        location = item.getp('clocation')
        if location == 'surface':
            # place the contours at the corresponding z level (contour3)
            pass
        elif location == 'base':
            if placement == 'bottom':
                # place the contours at the bottom (as in meshc or surfc)
                pass
            else:
                # standard contour plot
                pass

        if item.getp('clabels'):
            # add labels on the contour curves
            pass

        self._g.SetData2D('values%s' % name, z)
        self._g.Set('%s/data' % cntr, 'values%s' % name)

        legend = self._fix_latex(item.getp('legend'))
        if legend:
            self._g.Set('%s/key' % cntr, legend)
    
    def _add_vectors(self, item):
        if DEBUG:
            print "Adding vectors"
        # uncomment the following command if there is no support for
        # automatic scaling of vectors in the current plotting package:
        #item.scale_vectors()

        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # get line specifiactions (marker='.' means no marker):
        marker, color, style, width = self._get_linespecs(item)

        # scale the vectors according to this variable (scale=0 should
        # turn off automatic scaling):
        scale = item.getp('arrowscale')

        filled = item.getp('filledarrows') # draw filled arrows if True

        if z is not None and w is not None:
            # draw velocity vectors as arrows with components (u,v,w) at
            # points (x,y,z):
            pass
        else:
            # draw velocity vectors as arrows with components (u,v) at
            # points (x,y):
            pass

    def _add_streams(self, item):
        if DEBUG:
            print "Adding streams"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # starting positions for streams:
        sx, sy, sz = item.getp('startx'), item.getp('starty'), item.getp('startz')

        if item.getp('tubes'):
            # draw stream tubes from vector data (u,v,w) at points (x,y,z)
            n = item.getp('n') # no points along the circumference of the tube
            scale = item.getp('tubescale')
            pass
        elif item.getp('ribbons'):
            # draw stream ribbons from vector data (u,v,w) at points (x,y,z)
            width = item.getp('ribbonwidth')
            pass
        else:
            if z is not None and w is not None:
                # draw stream lines from vector data (u,v,w) at points (x,y,z)
                pass
            else:
                # draw stream lines from vector data (u,v) at points (x,y)
                pass
            pass

    def _add_isosurface(self, item):
        if DEBUG:
            print "Adding a isosurface"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume
        c = item.getp('cdata')  # pseudocolor data
        isovalue = item.getp('isovalue')

    def _add_slices(self, item):
        if DEBUG:
            print "Adding slices in a volume"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass
        pass

    def _add_contourslices(self, item):
        if DEBUG:
            print "Adding contours in slice planes"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels per plane
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass
        pass

    def _set_figure_size(self, fig):
        if DEBUG:
            print "Setting figure size"
        width, height = fig.getp('size')
        if width and height:
            # set figure width and height
            self._g.Set('/width', str(width))
            self._g.Set('/height', str(height))
        else:
            # use the default width and height in plotting package
            pass

    def figure(self, *args, **kwargs):
        # Extension of BaseClass.figure:
        # add a plotting package figure instance as fig._g and create a
        # link to it as self._g
        fig = BaseClass.figure(self, *args, **kwargs)
        try:
            fig._g
        except:
            # create plotting package figure and save figure instance
            # as fig._g
            if DEBUG:
                name = 'Fig ' + str(fig.getp('number'))
                print "creating figure %s in backend" % name

            name = 'Fig ' + str(self.getp('curfig'))
            fig._g = veusz.embed.Embedded(name)
            fig._g.EnableToolbar(enable=True)
            #fig._g.window.hide()
            
        self._g = fig._g  # link for faster access
        return fig
        
    def _replot(self):
        """Replot all axes and all plotitems in the backend."""
        # NOTE: only the current figure (gcf) is redrawn.
        if DEBUG:
            print "Doing replot in backend"

        fig = self.gcf()
        # add Veusz attributes to current figure (if not already added):
        try:
            fig._g
        except: 
            self.figure(self.getp('curfig'))
        
        self._g.To('/') # make sure we are at root before we start
        
        # reset the plotting package instance in fig._g now if needed
        try:
            self._g.Remove('/page1')
        except:
            pass
        self._g.To(self._g.Add('page'))
        
        self._set_figure_size(fig)

        i = 1
        nrows, ncolumns = fig.getp('axshape')
        if nrows != 1 or ncolumns != 1:
            self._g.To(self._g.Add('grid'))
            self._g.Set('rows', nrows)
            self._g.Set('columns', ncolumns)
            self._g.Set('leftMargin', '0.4cm')
            self._g.Set('rightMargin', '0.1cm')
            self._g.Set('topMargin', '0.7cm')
            self._g.Set('bottomMargin', '0.1cm')
        for axnr, ax in fig.getp('axes').items():
            legends = False
            pth = ax.getp('pth')
            if pth:
                # create axes in tiled position
                # this is subplot(nrows,ncolumns,pth)
                self._g.To('/page1/grid1')
            else:
                self._g.To('/page1')
            self._g.To(self._g.Add('graph'))
            plotitems = ax.getp('plotitems')
            plotitems.sort(self._cmpPlotProperties)
            for item in plotitems:
                name = str(i)
                func = item.getp('function') # function that produced this item
                if isinstance(item, Line):
                    self._add_line(item, name)
                elif isinstance(item, Surface):
                    self._add_surface(item, name,
                                      shading=ax.getp('shading'),
                                      cmap=ax.getp('colormap'))
                elif isinstance(item, Contours):
                    self._add_contours(item, name)
                elif isinstance(item, VelocityVectors):
                    self._add_vectors(item)
                elif isinstance(item, Streams):
                    self._add_streams(item)
                elif isinstance(item, Volume):
                    if func == 'isosurface':
                        self._add_isosurface(item)
                    elif func == 'slice_':
                        self._add_slices(item)
                    elif func == 'contourslice':
                        self._add_contourslices(item)
                legend = self._fix_latex(item.getp('legend'))
                if legend:
                    # add legend to plot
                    legends = True
                i += 1

            if legends:
                key = self._g.Add('key')
                self._g.Set('%s/horzPosn' % key, 'right')
                self._g.Set('%s/vertPosn' % key, 'top')
                self._g.Set('%s/Text/size' % key, str(ax.getp('fontsize')))
            self._set_axis_props(ax)
                    
        if self.getp('show'):
            # display plot on the screen
            if DEBUG:
                print "\nDumping plot data to screen\n"
                debug(self)
            #self._g.window.showNormal()
        else:
            #self._g.window.hide()
            pass

    def hardcopy(self, filename, **kwargs):
        """
        Supported extensions:

          '.eps'  (Encapsulated PostScript)
          '.svg'  (Scalable Vector Graphics)
          '.png'  (Portable Network Graphics)

        Optional keyword arguments:

          color -- True (colors) or False (gray-scale).
          
        """
        self.setp(**kwargs)
        color = self.getp('color')
        replot = kwargs.get('replot', True)
        if replot:
            self._replot()

        if DEBUG:
            print "Hardcopy to %s" % filename
            
        self._g.Export(filename, color=color)

    def clf(self):
        """Clear current figure."""
        self._g.Close()
        BaseClass.clf(self)

    def closefig(self, fignr=None):
        """Close figure window."""
        if not fignr: # no figure given, close current figure
            fignr = self._attrs['curfig']
        fig = self._figs[fignr]
        fig._g.Close()
        del fig._g

    def closefigs(self):
        """Close all figure windows."""
        for fig in self._figs.values():
            fig._g.Close()
            del fig._g
        BaseClass.closefigs(self)

    def save(self, filename):
        """Save the current Veusz settings to file."""
        self._g.Save(filename)

    # implement colormap functions here
    def jet(self, m=None):
        return 'spectrum'

    def hot(self, m=None):
        return 'heat'

    def gray(self, m=None):
        return 'grey'
    

    # Now we add the doc string from the methods in BaseClass to the
    # methods that are reimplemented in this backend:
    for cmd in BaseClass._matlab_like_cmds:
        if not '__' in cmd and hasattr(BaseClass, cmd):
            m1 = eval('BaseClass.%s' % cmd)
            try:
                m2 = eval('%s' % cmd)
            except NameError:
                pass
            else:
                if m1.__doc__ != m2.__doc__:
                    if m2.__doc__ is None:
                        m2.__doc__ = ""
                    m2.__doc__ = m1.__doc__ + m2.__doc__


plt = VeuszBackend()  # create backend instance
use(plt, globals())   # export public namespace of plt to globals()
backend = os.path.splitext(os.path.basename(__file__))[0][:-1]



"""Test LaTeX syntax in legends and titles."""
from scitools.std import *

def f1(t):
    return t**2*exp(-t**2)

def f2(t):
    return t**2*f1(t)

t = linspace(0, 3, 51)
y1 = f1(t)
y2 = f2(t)

plot(t, y1, 'r-', legend='t**2*exp(-t**2)',
     title='Testing legend with double mult for power')
savefig('tmp1.eps')
raw_input('Press Return key to quit: ')
import sys; sys.exit(0)
savefig('tmp1.eps')
savefig('tmp1.png')

figure()
plot(t, y1, 'r-', legend='t^2*exp(-t^2)',
     title='Testing legend with hat for power')
savefig('tmp2.eps')
savefig('tmp2.png')

figure()
plot(t, y1, 'r-.',
     t, y2, 'b--',
     legend=(r'$t^2\exp{(-t^2)}$', r'$t^4\exp{(-t^2)}$'),
     title=r'Testing real LaTeX syntax w/dollars: $\alpha$, $\beta + \Omega$')
savefig('tmp3.eps')
savefig('tmp3.png')

figure()
plot(t, y1, 'r-.',
     t, y2, 'b--',
     legend=(r't^2\exp{(-t^2)}', r't^4\exp{(-t^2)}'),
     title=r'Testing real LaTeX syntax (no dollars): \alpha, \beta + \Omega')
savefig('tmp4.eps')
savefig('tmp4.png')

raw_input('Press Return key to quit: ')



"""
Run all tests in the current directory.
"""

import os, sys
import unittest

def test_suite(level=1):
    suite = unittest.TestSuite()
    files = os.listdir(os.curdir)
    tests = []
    for f in files:
        if f.startswith('test_') and f.endswith('.py'):
            tests.append(f[:-3])

    for test in tests:
        m = __import__(test)
        if hasattr(m, 'test_suite'):
            suite.addTest(m.test_suite())
    return suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == '__main__':
    test()
    raw_input("press enter to exit")



#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.fr/access/helpdesk/help/techdoc/ref/streamparticles.html

from scitools.easyviz import *
from time import sleep
from scipy import io

wind = io.loadmat('wind_matlab_v6.mat')
x = wind['x']
y = wind['y']
z = wind['z']
u = wind['u']
v = wind['v']
w = wind['w']

setp(show=False)
sx, sy, sz = ndgrid([80]*36,seq(20,55,1),[5]*36)
sl = streamline(x,y,z,u,v,w,sx,sy,sz)
axis('tight')
view(30,30)
daspect([1,1,.125])
camproj('perspective')
camva(8)
box('on')
setp(show=True)
show()


figure()
# alternative syntax:
sl = streamline(x,y,z,u,v,w,sx,sy,sz,
                axis='tight',
                view=(30,30),
                daspect=[1,1,.125],
                camproj='perspective',
                camva=8,
                box='on')

raw_input('Press Return key to quit: ')




"""Test functions for legend command."""

from easyviztest import *


class test_legend_basic(EasyvizTestCase):
    def check_legend_simple(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        legend('sin(x)*x')
        title("plot(..);legend('sin(x)*x')")
        n()

    def check_legend_simple_kwarg(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,legend='sin(x)*x')
        title("plot(..,legend='sin(x)*x')")
        n()

    def check_legend_two_lines(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,x,v)
        legend('sin(x)*x','sin(x)*sqrt(x)')
        title("plot(..);legend('sin(x)*x','sin(x)*sqrt(x)'")
        n()

    def check_legend_two_lines_kwarg(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,x,v,legend=('sin(x)*x','sin(x)*sqrt(x)'))
        title("plot(..,legend=('sin(x)*x','sin(x)*sqrt(x)')")
        n()

    def check_legend_two_lines_with_hold(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        plot(x,y,'r:')
        legend('sin(x)*x')
        hold('on')
        plot(x,v,'b-o')
        legend('sin(x)*sqrt(x)')
        title("plot();legend();hold('on');plot();legend()")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_legend_three_lines(self):
        x,y,v,w = self.get_line_data()
        plot(y,v,w,x=x)
        legend('sin(x)*x','sin(x)*sqrt(x)','sin(x)*x**0.33333333')
        title("legend('sin(x)*x','sin(x)*sqrt(x)','sin(x)*x**0.33333333')")
        n()

    def check_legend_three_lines_kwarg(self):
        x,y,v,w = self.get_line_data()
        plot(y,v,w,x=x,
             legend=('sin(x)*x','sin(x)*sqrt(x)','sin(x)*x**0.33333333'))
        title("plot(..,legend=('sin(x)*x','sin(x)*sqrt(x)','sin(x)*x**0.33333333'))")
        n()

    def check_legend_multiple_lines(self):
        format = self.get_format_string_data()
        m = len(format)
        x = linspace(0,1,m)
        setp(show=False)
        for i in range(1,m+1):
            y = linspace(i,m/2.0,m)
            plot(x,y,format[i-1],hold='on')
            legend('line%d' % i)
        title("legends on multiple lines")
        hold('off')
        setp(show=screenplot)
        show()
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_legend_basic,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""Test functions for mesh command."""

# Same as surf command. See test_surf.py for more tests.

from easyviztest import *


class test_mesh_basic(EasyvizTestCase):
    def check_mesh_default(self):
        x,y,xv,yv,values = self.get_2D_data()
        mesh(xv,yv,values)
        title("mesh(xv,yv,values)")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_mesh_basic,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



#!/usr/bin/env python

# Examples taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/ref/isosurface.html and
# http://www.mathworks.com/access/helpdesk/help/techdoc/visualize/f5-3653.html

from scitools.easyviz import *
from time import sleep

setp(interactive=False)

x, y, z, v = flow()

h = isosurface(x,y,z,v,-3)
#setp(h,'FaceColor','red','EdgeColor','none');
h.setp(opacity=0.5)
shading('interp')
daspect([1,1,1])
view(3)
axis('tight')
#camlight()
#lighting('gouraud')
show()

#hardcopy('tmp_isosurf1a.eps')
#hardcopy('tmp_isosurf1a_lq.eps', vector_file=False)
#hardcopy('tmp_isosurf1a.png')

figure()
h = isosurface(x,y,z,v,0)
#setp(hpatch,'FaceColor','red','EdgeColor','none')
shading('interp')
daspect([1,4,4])
view([-65,20])
axis('tight')
#camlight('left')
#setp(gcf,'Renderer','zbuffer');
#lighting('phong')
show()
#sleep(3)
raw_input('Press Return key to quit: ')

#hardcopy('tmp_isosurf1b.eps')
#hardcopy('tmp_isosurf1b_lq.eps', vector_file=False)
#hardcopy('tmp_isosurf1b.png')
